﻿/* - TextLCD full -
 * This example creates a nice gui to display the connected TextLCD's details as well as provides fields
 * and checkboxes to manipulate all of the available controls and methods for a TextLCD such as backligh,
 * cursor, cursor blink, contrast, and a simulation of the custom characters ability of the TextLCD api.
 *
 * NOTE: You can only create and display a maximum of 8 custom characters at a time on the TextLCD.
 * The method to create a custom character is not simply straight bit setting.  There is a web tool
 * located in the documentation section of the website that one can refer to in order to generate the
 * int values to generate a custom character
 *
 * Please note that this example was designed to work with only one Phidget TextLCD connected.
 * For an example showing how to use two Phidgets of the same time concurrently, please see the
 * Servo-multi example in the Servo Examples.
 *
 * Copyright 2007 Phidgets Inc.
 * This work is licensed under the Creative Commons Attribution 2.5 Canada License.
 * To view a copy of this license, visit http://creativecommons.org/licenses/by/2.5/ca/
 */

using Phidget22;
using Phidget22.Events;
using Phidget22.ExampleUtils;
using System;
using System.Drawing;
using System.Linq;
using System.Windows.Forms;

namespace TextLCD_Example {
	public partial class Form1 : Form {

		CommandLineOpen openArgs;
		LCD openCh = null;
		formCleaner cleaner;
		private LCD lcd; //Declare a textLCD object
		private ErrorEventBox errorBox;

		public Form1() {
			InitializeComponent();
			openArgs = new CommandLineOpen(this);
			cleaner = new formCleaner(this);
		}

		public Form1(LCD ch) {
			InitializeComponent();
			openCh = ch;
			cleaner = new formCleaner(this);
		}

		//initialize our TextLCD Phidget and hook the event handlers
		private void Form1_Load(object sender, EventArgs e) {


			errorBox = new ErrorEventBox(this);

			screenSizeCombo.DataSource = typeof(LCDScreenSize).ToList();
			screenSizeCombo.SelectedIndex = -1;

			lcd = openCh ?? openArgs.makeChannel<LCD>();

			lcd.Attach += lcd_Attach;
			lcd.Detach += lcd_Detach;
			lcd.Error += lcd_Error;

			try {
				lcd.Open();
			} catch (PhidgetException ex) { errorBox.addMessage("Error opening device: " + ex.Message); }
		}

		//When the application is terminating, close the Phidget.
		private void Form1_FormClosing(object sender, FormClosingEventArgs e) {

			lcd.Attach -= lcd_Attach;
			lcd.Detach -= lcd_Detach;
			lcd.Error -= lcd_Error;
			lcd.Close();
		}

		//attach event handler...we will display the attach status and the attached TextLCD device's details
		//We will also enable the fields used to manipulate the TextLCD as well as do some initializing of
		//ranges and maximum values for some of these fields
		void lcd_Attach(object sender, AttachEventArgs e) {

			phidgetInfoBox1.FillPhidgetInfo((Phidget)sender);
			settingsBox.Visible = true;

			LCD attachedDevice = (LCD)sender;

			setupTxtInputs();

			cursorChk.Checked = false;
			cursorBlinkChk.Checked = false;
			customChrChk.Checked = false;

			try {
				contrastTrkBr.Value = (int)(attachedDevice.Contrast * contrastTrkBr.Maximum);
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error reading contrast: " + ex.Message);
			}

			try {
				attachedDevice.Backlight = attachedDevice.MaxBacklight;
				brightnessTrkBr.Value = (int)(attachedDevice.Backlight * brightnessTrkBr.Maximum);
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error reading brightness: " + ex.Message);
			}

			switch (attachedDevice.DeviceID) {

			case DeviceID.PN_1204:
				try {
					// Screen Size is a must set on 1204
					attachedDevice.ScreenSize = LCDScreenSize.NoScreen;
				} catch { }
				screenSizeCombo.Enabled = true;
				initBtn.Visible = true;
				break;

			default:
				screenSizeCombo.Enabled = false;
				initBtn.Visible = false;
				break;
			}

			try {
				screenSizeCombo.SelectedValue = attachedDevice.ScreenSize;
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error reading screen size: " + ex.Message);
			}
		}

		//detach event handler.... we will display the device attach status and clear all the other fields
		void lcd_Detach(object sender, DetachEventArgs e) {

			phidgetInfoBox1.Clear();
			cleaner.clean();
			settingsBox.Visible = false;
		}

		//Error event handler..we will simply display the error details text in a popup message box
		void lcd_Error(object sender, ErrorEventArgs e) {

			errorBox.addMessage(e.Description);
		}

		private void writeText(string txt, int row) {

			if (clearingTxtInputs)
				return;

			try {
				// We write twice so the cursor shows up in the expected location
				lcd.WriteText(LCDFont.Dimensions_5x8, 0, row, "".PadRight(lcd.Width));
				lcd.WriteText(LCDFont.Dimensions_5x8, 0, row, txt);
				lcd.Flush();
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting row " + row + ": " + ex.Message);
			}
		}

		//line 1 textChanged event handler...when the user is entering text, pass it on to the textLCD to be displayed
		private void dispText1_TextChanged(object sender, EventArgs e) {

			writeText(dispText1.Text, 0);
		}

		private void dispText2_TextChanged(object sender, EventArgs e) {

			writeText(dispText2.Text, 1);
		}

		private void dispText3_TextChanged(object sender, EventArgs e) {

			writeText(dispText3.Text, 2);
		}

		private void dispText4_TextChanged(object sender, EventArgs e) {

			writeText(dispText4.Text, 3);
		}

		//If the clear button is clicked, we want to clear the fields as well as clear the TextLCD screen
		private void clearBtn_Click(object sender, EventArgs e) {

			try {
				lcd.Clear();
				lcd.Flush();
				clearTxtInputs();
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error clearing screen: " + ex.Message);
			}
		}

		//display ou simulation of the custom characters when the checkbox is clicked
		private void customChrChk_CheckedChanged(object sender, EventArgs e) {

			try {
				if (customChrChk.Checked) {
					Byte[] smileyface = getBitmapBytes(
						  "          "
						+ "  ██  ██  "
						+ "          "
						+ "    ██    "
						+ "          "
						+ "██      ██"
						+ "  ██████  "
						+ "          ");

					Byte[] sadface = getBitmapBytes(
						  "          "
						+ "  ██  ██  "
						+ "          "
						+ "    ██    "
						+ "          "
						+ "  ██████  "
						+ "██      ██"
						+ "          ");

					Byte[] musicNotes = getBitmapBytes(
						  "        ██"
						+ "      ████"
						+ "    ██  ██"
						+ "  ██    ██"
						+ "  ██    ██"
						+ "  ██  ████"
						+ "████  ████"
						+ "████      ");

					Byte[] bell = getBitmapBytes(
						  "    ██    "
						+ "  ██████  "
						+ "  ██████  "
						+ "  ██████  "
						+ "██████████"
						+ "          "
						+ "    ██    "
						+ "          ");

					Byte[] heart = getBitmapBytes(
						  "          "
						+ "  ██  ██  "
						+ "██████████"
						+ "██████████"
						+ "  ██████  "
						+ "    ██    "
						+ "          "
						+ "          ");

					//lcd.SetCharacterBitmap(LCDFont.Dimensions_5x8, "☺", smileyface);
					lcd.SetCharacterBitmap(LCDFont.Dimensions_5x8, "\x8", smileyface);
					lcd.SetCharacterBitmap(LCDFont.Dimensions_5x8, "\x9", sadface);
					lcd.SetCharacterBitmap(LCDFont.Dimensions_5x8, "\xA", musicNotes);
					lcd.SetCharacterBitmap(LCDFont.Dimensions_5x8, "\xB", bell);
					lcd.SetCharacterBitmap(LCDFont.Dimensions_5x8, "\xC", heart);

					lcd.Clear();
					lcd.WriteText(LCDFont.Dimensions_5x8, 0, 0, "Custom \x8\x9\x8\x9 \xA \xB \xC");
					lcd.WriteText(LCDFont.Dimensions_5x8, 0, 1, "Characters! \x8\x9\x8\x9");
					if (lcd.Height == 4) {
						lcd.WriteText(LCDFont.Dimensions_5x8, 1, 2, "Custom \x8\x9\x8\x9 \xA \xB \xC");
						lcd.WriteText(LCDFont.Dimensions_5x8, 1, 3, "Characters! \x8\x9\x8\x9");
					}
					lcd.Flush();
				} else {
					lcd.Clear();
					lcd.Flush();
				}
				clearTxtInputs();
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting custom characters: " + ex.Message);
			}
		}

		//adjust the contast of the screen on the textLCD based on the value selected on the slider (100 to 0)
		private void contrastTrkBr_Scroll(object sender, EventArgs e) {

			try {
				lcd.Contrast = contrastTrkBr.Value / (double)contrastTrkBr.Maximum;
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting contrast: " + ex.Message);
			}
		}

		private void brightnessTrkBr_Scroll(object sender, EventArgs e) {

			try {
				lcd.Backlight = brightnessTrkBr.Value / (double)brightnessTrkBr.Maximum;
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting backlight: " + ex.Message);
			}
		}

		private void cursorChk_CheckedChanged(object sender, EventArgs e) {

			try {
				lcd.CursorOn = cursorChk.Checked;
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting cursor: " + ex.Message);
			}
		}

		private void cursorBlinkChk_CheckedChanged(object sender, EventArgs e) {

			try {
				lcd.CursorBlink = cursorBlinkChk.Checked;
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting cursor blink: " + ex.Message);
			}
		}

		private byte[] getBitmapBytes(String charStr) {

			Byte[] bytes = new byte[charStr.Length / 2];
			for (int i = 0; i < charStr.Length / 2; i++)
				bytes[i] = (byte)(charStr.ToArray()[i * 2] == ' ' ? 0x00 : 0x01);
			return bytes;
		}

		private void setupTxtInputs() {

			dispText1.Enabled = false;
			dispText2.Enabled = false;
			dispText3.Enabled = false;
			dispText4.Enabled = false;

			dispText1.BackColor = SystemColors.InactiveCaption;
			dispText2.BackColor = SystemColors.InactiveCaption;
			dispText3.BackColor = SystemColors.InactiveCaption;
			dispText4.BackColor = SystemColors.InactiveCaption;

			if (lcd.Height >= 1) {
				dispText1.Enabled = true;
				dispText1.BackColor = SystemColors.Window;
				dispText1.MaxLength = lcd.Width;
			}
			if (lcd.Height >= 2) {
				dispText2.Enabled = true;
				dispText2.BackColor = SystemColors.Window;
				dispText2.MaxLength = lcd.Width;
			}
			if (lcd.Height >= 3) {
				dispText3.Enabled = true;
				dispText3.BackColor = SystemColors.Window;
				dispText3.MaxLength = lcd.Width;
			}
			if (lcd.Height >= 4) {
				dispText4.Enabled = true;
				dispText4.BackColor = SystemColors.Window;
				dispText4.MaxLength = lcd.Width;
			}
		}

		private bool clearingTxtInputs = false;
		private void clearTxtInputs() {

			clearingTxtInputs = true;
			dispText1.Clear();
			dispText2.Clear();
			dispText3.Clear();
			dispText4.Clear();
			clearingTxtInputs = false;
		}

		private void screenSizeCombo_SelectedIndexChanged(object sender, EventArgs e) {

			if (((ComboBox)sender).SelectedIndex == -1 || lcd == null)
				return;

			try {
				if (lcd.DeviceID != DeviceID.PN_1204)
					return;
				lcd.ScreenSize = (LCDScreenSize)Enum.Parse(typeof(LCDScreenSize), screenSizeCombo.SelectedValue.ToString());
				setupTxtInputs();
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting screen size: " + ex.Message);
			}
		}

		private void initBtn_Click(object sender, EventArgs e) {

			try {
				lcd.Initialize();
				clearTxtInputs();
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error initializing screen: " + ex.Message);
			}
		}
	}
}